from time import sleep

import boto3

TERMINATION_REASON = "Termination requested by workflow."
"""Reason sent for terminating jobs from a workflow."""


def terminate_batch_job(job_arn: str) -> None:
    """
    Terminate job on AWS Batch.

    Task will sleep for 1 minute after sending the termination request.

    Parameters
    ----------
    job_arn
        Job ARN.
    """

    client = boto3.client("batch")
    client.terminate_job(jobId=job_arn, reason=TERMINATION_REASON)
    sleep(60)



from docker import APIClient


def submit_docker_job(api_client: APIClient, job_definition: dict, volume_name: str) -> str:
    """
    Submit Docker job.

    Parameters
    ----------
    api_client
        Docker API client.
    job_definition
        Docker job definition used to create job container.
    volume_name
        Name of the docker volume.

    Returns
    -------
    :
        Container ID.
    """

    host_config = api_client.create_host_config(binds={volume_name: {"bind": "/mnt", "mode": "rw"}})

    container = api_client.create_container(**job_definition, host_config=host_config)
    container_id = container.get("Id")

    api_client.start(container=container_id)

    return container_id



"""Tasks for working with AWS ECS Fargate."""

from prefect import task

from .check_fargate_task import check_fargate_task
from .make_fargate_task import make_fargate_task
from .register_fargate_task import register_fargate_task
from .submit_fargate_task import submit_fargate_task
from .terminate_fargate_task import terminate_fargate_task

check_fargate_task = task(check_fargate_task)
make_fargate_task = task(make_fargate_task)
register_fargate_task = task(register_fargate_task)
submit_fargate_task = task(submit_fargate_task)
terminate_fargate_task = task(terminate_fargate_task)



import boto3
from deepdiff import DeepDiff


def register_batch_job(job_definition: dict) -> str:
    """
    Register job definition to AWS Batch.

    If a definition for the given job definition name already exists, and the
    contents of the definition are not changed, then the method will return the
    existing job definition ARN rather than creating a new revision.

    Parameters
    ----------
    job_definition
        Batch job definition.

    Returns
    -------
    :
        Job definition ARN.
    """

    client = boto3.client("batch")
    response = client.describe_job_definitions(
        jobDefinitionName=job_definition["jobDefinitionName"],
    )

    if len(response["jobDefinitions"]) > 0:
        existing_definition = sorted(response["jobDefinitions"], key=lambda d: d["revision"]).pop()
        diff = DeepDiff(job_definition, existing_definition, ignore_order=True)

        if "values_changed" not in diff:
            return existing_definition["jobDefinitionArn"]

    response = client.register_job_definition(**job_definition)
    return response["jobDefinitionArn"]



from time import sleep
from typing import Union

import boto3
from prefect.context import TaskRunContext
from prefect.states import Failed, State

RETRIES_EXCEEDED_EXIT_CODE = 80
"""Exit code used when task run retries exceed the maximum retries."""


def check_batch_job(job_arn: str, max_retries: int) -> Union[int, State, bool]:
    """
    Check for exit code of an AWS Batch job.

    If this task is running within a Prefect flow, it will use the task run
    context to get the current run count. While the run count is below the
    maximum number of retries, the task will continue to attempt to get the exit
    code, and can be called with a retry delay to periodically check the status
    of jobs.

    If this task is not running within a Prefect flow, the ``max_retries``
    parameters is ignored. Jobs that are still running will throw an exception.

    Parameters
    ----------
    job_arn
        Job ARN.
    max_retries
        Maximum number of retries.

    Returns
    -------
    :
        Exit code if the job is complete, otherwise throws an exception.
    """

    context = TaskRunContext.get()

    if context is not None and context.task_run.run_count > max_retries:
        return RETRIES_EXCEEDED_EXIT_CODE

    client = boto3.client("batch")
    response = client.describe_jobs(jobs=[job_arn])["jobs"]

    # Job responses are not immediately available. Wait until available.
    while len(response) != 1:
        sleep(10)
        response = client.describe_jobs(jobs=[job_arn])["jobs"]

    status = response[0]["status"]

    # Wait until job is running or completed.
    while status not in ("RUNNING", "SUCCEEDED", "FAILED"):
        sleep(10)
        response = client.describe_jobs(jobs=[job_arn])["jobs"]
        status = response[0]["status"]

    # For jobs that are running, throw the appropriate exception.
    if context is not None and status == "RUNNING":
        return Failed()
    if status == "RUNNING":
        raise RuntimeError("Job is in RUNNING state and does not have exit code.")

    exitcode = response[0]["attempts"][0]["container"]["exitCode"]
    return exitcode



"""Tasks for working with manifest files."""

from prefect import task

from .filter_manifest_files import filter_manifest_files
from .find_missing_conditions import find_missing_conditions
from .summarize_manifest_files import summarize_manifest_files
from .update_manifest_contents import update_manifest_contents

filter_manifest_files = task(filter_manifest_files)
find_missing_conditions = task(find_missing_conditions)
summarize_manifest_files = task(summarize_manifest_files)
update_manifest_contents = task(update_manifest_contents)



"""Tasks for working with Docker."""

from prefect import task

from .check_docker_job import check_docker_job
from .clean_docker_job import clean_docker_job
from .create_docker_volume import create_docker_volume
from .get_docker_logs import get_docker_logs
from .make_docker_job import make_docker_job
from .remove_docker_volume import remove_docker_volume
from .run_docker_command import run_docker_command
from .submit_docker_job import submit_docker_job
from .terminate_docker_job import terminate_docker_job

check_docker_job = task(check_docker_job)
clean_docker_job = task(clean_docker_job)
create_docker_volume = task(create_docker_volume)
get_docker_logs = task(get_docker_logs)
make_docker_job = task(make_docker_job)
remove_docker_volume = task(remove_docker_volume)
run_docker_command = task(run_docker_command)
submit_docker_job = task(submit_docker_job)
terminate_docker_job = task(terminate_docker_job)



from docker import APIClient


def get_docker_logs(api_client: APIClient, container_id: str, log_filter: str) -> str:
    """
    Get logs for Docker job.

    Parameters
    ----------
    api_client
        Docker API client.
    container_id
        Docker container ID.
    log_filter
        Filter for log events (use "-" for exclusion).

    Returns
    -------
    :
        All filtered log events.
    """

    logs = api_client.logs(container=container_id).decode("utf-8")

    log_items = logs.split("\n")
    if "-" in log_filter:
        log_filter = log_filter.replace("-", "")
        log_items = [item for item in log_items if log_filter not in item]
    else:
        log_items = [item for item in log_items if log_filter in item]

    return "\n".join(log_items)



import boto3

LOG_GROUP_NAME = "/aws/batch/job"
"""AWS Batch log group name."""


def get_batch_logs(job_arn: str, log_filter: str) -> str:
    """
    Get logs for AWS Batch job.

    Parameters
    ----------
    job_arn
        Job ARN.
    log_filter
        Filter for log events.

    Returns
    -------
    :
        All filtered log events.
    """

    client = boto3.client("batch")
    response = client.describe_jobs(jobs=[job_arn])["jobs"][0]
    log_stream = response["container"]["logStreamName"]

    client = boto3.client("logs")
    log_events: list[str] = []

    response = client.filter_log_events(
        logGroupName=LOG_GROUP_NAME,
        logStreamNames=[log_stream],
        filterPattern=log_filter,
    )

    if response["events"]:
        log_events = log_events + [event["message"] for event in response["events"]]

    while "nextToken" in response:
        response = client.filter_log_events(
            logGroupName="/aws/batch/job",
            logStreamNames=[log_stream],
            filterPattern=log_filter,
            nextToken=response["nextToken"],
        )

        log_events = log_events + [event["message"] for event in response["events"]]

    return "\n".join(log_events)



import boto3
from deepdiff import DeepDiff


def register_fargate_task(task_definition: dict) -> str:
    """
    Register task definition to ECS Fargate.

    If a definition for the given task definition name already exists, and the
    contents of the definition are not changed, then the method will return the
    existing task definition ARN rather than creating a new revision.

    Parameters
    ----------
    task_definition
        Fargate task definition.

    Returns
    -------
    :
        Task definition ARN.
    """

    client = boto3.client("ecs")
    response = client.list_task_definitions(familyPrefix=task_definition["family"])

    if len(response["taskDefinitionArns"]) > 0:
        response = client.describe_task_definition(taskDefinition=task_definition["family"])
        existing_definition = response["taskDefinition"]
        diff = DeepDiff(task_definition, existing_definition, ignore_order=True)

        if "values_changed" not in diff:
            return existing_definition["taskDefinitionArn"]

    response = client.register_task_definition(**task_definition)
    return response["taskDefinition"]["taskDefinitionArn"]



import os

import pandas as pd


def update_manifest_contents(location_keys: dict) -> pd.DataFrame:
    """
    Update manifest using files at given keys at specified locations.

    Parameters
    ----------
    location_keys
        Map of locations to list of file keys.

    Returns
    -------
    :
        Combined manifest of file keys, extensions, and locations.
    """

    all_manifests = []

    for location, keys in location_keys.items():
        location_manifest = make_file_manifest(location, keys)
        all_manifests.append(location_manifest)

    if len(all_manifests) == 0:
        return pd.DataFrame(columns=["KEY", "EXTENSION", "LOCATION", "FULL_KEY"])

    manifest = pd.concat(all_manifests)
    manifest.sort_values(by=["EXTENSION", "KEY"], inplace=True)
    manifest.reset_index(drop=True, inplace=True)

    return manifest


def make_file_manifest(location: str, keys: list[str]) -> pd.DataFrame:
    """
    Create manifest for location with given list of file keys.

    Parameters
    ----------
    location
        File location (local path or S3 bucket).
    keys
        List of file keys.

    Returns
    -------
    :
        Manifest of file keys, extensions, and locations.
    """

    contents = []

    for key in keys:
        short_key = os.path.split(key)[1].split(".")[0]
        extension = ".".join(os.path.split(key)[1].split(".")[1:])
        contents.append((short_key, extension, location, key))

    manifest = pd.DataFrame(contents, columns=["KEY", "EXTENSION", "LOCATION", "FULL_KEY"])

    return manifest



from jinja2 import StrictUndefined, Template


def generate_input_contents(template: str, conditions: list[dict]) -> list[str]:
    """
    Generate input contents from template and conditions.

    Parameters
    ----------
    template
        Template string.
    conditions
        List of conditions.

    Returns
    -------
    :
        List of rendered templates.
    """

    compiled_template = Template(template, undefined=StrictUndefined)
    rendered_templates = [compiled_template.render(condition) for condition in conditions]
    return rendered_templates



from docker import APIClient


def clean_docker_job(api_client: APIClient, container_id: str) -> None:
    """
    Clean up container if it is not currently running.

    Parameters
    ----------
    api_client
        Docker API client.
    container_id
        ID of container to remove.
    """

    status = api_client.containers(all=True, filters={"id": container_id})[0]["State"]

    if status != "running":
        api_client.remove_container(container=container_id)



from typing import Any

import boto3


def submit_batch_job(
    name: str,
    job_definition_arn: str,
    user: str,
    queue: str,
    size: int,
    **kwargs: Any,
) -> list[str]:
    """
    Submit AWS Batch job.

    Parameters
    ----------
    name
        Job name.
    job_definition_arn
        Job definition ARN.
    user
        User name prefix for job name.
    queue
        Job queue.
    size
        Number of jobs in array.
    **kwargs
        Additional parameters for job submission. The keyword arguments are
        passed to `boto3` Batch client method `submit_job`.


    Returns
    -------
    :
        List of job ARNs.
    """

    default_job_submission = {
        "jobName": f"{user}_{name}",
        "jobQueue": queue,
        "jobDefinition": job_definition_arn,
    }

    if size > 1:
        default_job_submission["arrayProperties"] = {"size": size}  # type: ignore

    client = boto3.client("batch")
    job_submission = default_job_submission | kwargs
    response = client.submit_job(**job_submission)

    if size > 1:
        return [f"{response['jobArn']}:{i}" for i in range(size)]

    return [response["jobArn"]]



from time import sleep

import boto3

TERMINATION_REASON = "Termination requested by workflow."
"""Reason sent for terminating jobs from a workflow."""


def terminate_fargate_task(cluster: str, task_arn: str) -> None:
    """
    Terminate task on AWS Fargate.

    Parameters
    ----------
    cluster
        ECS cluster name.
    task_arn
        Task ARN.
    """

    client = boto3.client("ecs")
    client.stop_task(cluster=cluster, task=task_arn, reason=TERMINATION_REASON)
    sleep(60)



"""Tasks for working with AWS Batch."""

from prefect import task

from .check_batch_job import check_batch_job
from .get_batch_logs import get_batch_logs
from .make_batch_job import make_batch_job
from .register_batch_job import register_batch_job
from .submit_batch_job import submit_batch_job
from .terminate_batch_job import terminate_batch_job

check_batch_job = task(check_batch_job)
get_batch_logs = task(get_batch_logs)
make_batch_job = task(make_batch_job)
register_batch_job = task(register_batch_job)
submit_batch_job = task(submit_batch_job)
terminate_batch_job = task(terminate_batch_job)



import re

import pandas as pd
from tabulate import tabulate


def summarize_manifest_files(
    manifest: pd.DataFrame, name: str, conditions: list[dict], seeds: list[int]
) -> str:
    """
    Summarize manifest files into a table.

    Summary table is formatted as:

    .. code-block:: bash

        ┍━━━━━━━━━━━━━━━━━┯━━━━━━━━━━━━━━━┯━━━━━┯━━━━━━━━━━━━━┑
        │                 │ extension.a   │ ... │ extension.n │
        ┝━━━━━━━━━━━━━━━━━┿━━━━━━━━━━━━━━━┿━━━━━┿━━━━━━━━━━━━━┥
        │ condition_key_a │ #/# (##.## %) │ ... │ ✓           │
        │ condition_key_b │ #/# (##.## %) │ ... │ ✓           │
        │ ...             │...            │ ... │ ...         │
        │ condition_key_n │ #/# (##.## %) │ ... │ ✓           │
        ┕━━━━━━━━━━━━━━━━━┷━━━━━━━━━━━━━━━┷━━━━━┷━━━━━━━━━━━━━┙

    For file extensions that have files  or each random seed, the summary table
    reports the number and percentage of random seeds. For file extensions with
    only one file per conditions, a checkmark (✓) is used to indicate if the
    file exists or not.

    Parameters
    ----------
    manifest
        Manifest of file keys, extensions, and locations.
    name
        Name of the simulation series.
    conditions
        List of series condition dictionaries (must include unique condition
        "key").
    seeds
        List of series random seeds.

    Returns
    -------
    :
        Manifest summary table.
    """

    condition_keys = [f"{name}_{condition['key']}" for condition in conditions]
    manifest_keys = manifest.set_index("KEY").filter(regex=f"^{name}", axis="index").reset_index()
    extensions = manifest_keys["EXTENSION"].unique()

    counts = pd.DataFrame(index=condition_keys, columns=extensions).fillna(0)
    summary = pd.DataFrame(index=condition_keys, columns=extensions)

    for entry in manifest_keys.to_dict("records"):
        key = entry["KEY"]
        extension = entry["EXTENSION"]

        match = re.search("[0-9]{4}", key)
        if match:
            key = entry["KEY"].replace(f"_{match.group(0)}", "")
            if key in condition_keys and int(match.group(0)) in seeds:
                counts.loc[key][extension] += 1
                count = counts.loc[key][extension]
                percent = count / len(seeds) * 100
                summary.loc[key, extension] = f"{count}/{len(seeds)} ({percent:.2f} %)"
        elif key in condition_keys:
            summary.loc[key, extension] = "✓"

    summary.dropna(axis=1, how="all", inplace=True)
    summary.fillna("", inplace=True)

    return tabulate(summary, headers="keys", tablefmt="mixed_outline")



from docker import APIClient


def create_docker_volume(api_client: APIClient, path: str) -> dict:
    """
    Create a docker volume that copies content to specified path.

    Parameters
    ----------
    api_client
        Docker API client.
    path
        Local path for volume.

    Returns
    -------
    :
        Created volume reference object.
    """

    return api_client.create_volume(
        driver="local", driver_opts={"type": "none", "device": path, "o": "bind"}
    )



from fnmatch import fnmatch

import pandas as pd


def filter_manifest_files(
    manifest: pd.DataFrame,
    extensions: list[str],
    include_filters: list[str],
    exclude_filters: list[str],
) -> dict:
    """
    Filter manifest file keys by incomplete extensions and given filters.

    Parameters
    ----------
    manifest
        Manifest of file keys, extensions, and locations.
    extensions
        List of single simulation output extensions.
    include_filters
        List of Unix filename pattern matching filter for included file keys.
    exclude_filters
        List of Unix filename pattern matching filter for excluded file keys.

    Returns
    -------
    :
        Map of filtered manifest file keys to extensions and locations.
    """

    complete_manifest = filter_incomplete_extensions(manifest, extensions)
    manifest_files = convert_to_dictionary(complete_manifest)
    selected_keys = filter_file_keys(list(manifest_files.keys()), include_filters, exclude_filters)

    return {key: manifest_files[key] for key in selected_keys}


def filter_incomplete_extensions(manifest: pd.DataFrame, extensions: list[str]) -> pd.DataFrame:
    """
    Filters manifest for files with incomplete set of extensions.

    Parameters
    ----------
    manifest
        Manifest of file keys, extensions, and locations.
    extensions
        List of single simulation output extensions.

    Returns
    -------
    :
        Filtered manifest of file keys, extensions, and locations.
    """

    filtered = manifest.groupby("KEY").filter(
        lambda x: len(set(extensions) - set(x["EXTENSION"])) == 0
    )
    return filtered


def convert_to_dictionary(manifest: pd.DataFrame) -> dict:
    """
    Convert manifest dataframe to map of file key to extensions and locations.

    Parameters
    ----------
    manifest
        Manifest of file keys, extensions, and locations.

    Returns
    -------
    :
        Map of file key to extensions and locations.
    """

    # Return empty dictionary if there are no entries.
    if manifest.empty:
        return {}

    # Convert into dictionary of dictionaries.
    manifest_dict = {}

    for key, group in manifest.groupby("KEY"):
        converted = group.to_dict("records")
        manifest_dict[key] = {
            x["EXTENSION"]: {"location": x["LOCATION"], "key": x["FULL_KEY"]} for x in converted
        }

    return manifest_dict


def filter_file_keys(
    files: list[str], include_filters: list[str], exclude_filters: list[str]
) -> set:
    """
    Filter keys using include and exclude Unix filename pattern matching filters.

    Parameters
    ----------
    files
        List of file keys.
    include_filters
        List of Unix filename pattern matching filter for included file keys.
    exclude_filters
        List of Unix filename pattern matching filter for excluded file keys.

    Returns
    -------
    :
        Filtered set of file keys.
    """

    # Filter key list by include and exclude filters.
    selected_keys = set()
    unselected_keys = set()

    # Filter files for matches to include filters.
    for include in include_filters:
        selected_keys.update([key for key in files if fnmatch(key, include)])

    # Filter files for matches to exclude filters.
    for exclude in exclude_filters:
        unselected_keys.update([key for key in files if fnmatch(key, exclude)])

    # Remove unselected keys from selected keys.
    return selected_keys - unselected_keys



from typing import Optional


def make_batch_job(
    name: str,
    image: str,
    vcpus: int,
    memory: int,
    environment: Optional[list[dict[str, str]]] = None,
    job_role_arn: Optional[str] = None,
) -> dict:
    """
    Create batch job definition.

    Docker images on the Docker Hub registry are available by default, and can
    be specified using ``image:tag``. Otherwise, use ``repository/image:tag``.

    Environment variables are passed as key-value pairs using the following
    structure:

    .. code-block:: python

        [
            { "name" : "envName1", "value" : "envValue1" },
            { "name" : "envName2", "value" : "envValue2" },
            ...
        ]

    Parameters
    ----------
    name
        Job definition name.
    image
        Docker image.
    vcpus
        Number of vCPUs to reserve for the container.
    memory
        Memory limit available to the container
    environment
        List of environment variables as key-value pairs.
    job_role_arn
        ARN for IAM role for the job container.

    Returns
    -------
    :
        Job definition.
    """

    container_properties = {
        "image": image,
        "vcpus": vcpus,
        "memory": memory,
    }

    if environment is not None:
        container_properties["environment"] = environment

    if job_role_arn is not None:
        container_properties["jobRoleArn"] = job_role_arn

    return {
        "jobDefinitionName": name,
        "type": "container",
        "containerProperties": container_properties,
    }



def make_fargate_task(
    name: str,
    image: str,
    region: str,
    user: str,
    vcpus: int,
    memory: int,
    task_role_arn: str,
    execution_role_arn: str,
) -> dict:
    """
    Create Fargate task definition.

    Docker images on the Docker Hub registry are available by default, and can
    be specified using ``image:tag``. Otherwise, use ``repository/image:tag``.

    Parameters
    ----------
    name
        Task definition name.
    image
        Docker image.
    region
        Logging region.
    user
        User name prefix for task name.
    vcpus
        Hard limit of CPU units to present to the task.
    memory
        Hard limit of memory to present to the task.
    task_role_arn
        ARN for IAM role for the task container.
    execution_role_arn : str
        ARN for IAM role for the container agent.

    Returns
    -------
    :
        Task definition.
    """

    return {
        "containerDefinitions": [
            {
                "name": f"{user}_{name}",
                "image": image,
                "essential": True,
                "portMappings": [],
                "environment": [],
                "mountPoints": [],
                "volumesFrom": [],
                "logConfiguration": {
                    "logDriver": "awslogs",
                    "options": {
                        "awslogs-group": f"/ecs/{user}_{name}",
                        "awslogs-region": region,
                        "awslogs-stream-prefix": "ecs",
                        "awslogs-create-group": "true",
                    },
                },
            }
        ],
        "family": f"{user}_{name}",
        "networkMode": "awsvpc",
        "requiresCompatibilities": ["FARGATE"],
        "taskRoleArn": task_role_arn,
        "executionRoleArn": execution_role_arn,
        "cpu": str(vcpus),
        "memory": str(memory),
    }



import pandas as pd


def find_missing_conditions(
    manifest: pd.DataFrame,
    name: str,
    conditions: list[dict],
    seeds: list[int],
    extensions: list[str],
) -> list[dict]:
    """
    Find simulations missing or incomplete from list of conditions.

    Parameters
    ----------
    manifest
        Manifest of all files for simulation series.
    name
        Name of the simulation series.
    conditions
        List of series condition dictionaries (must include unique condition
        "key").
    seeds
        List of series random seeds.
    extensions
        List of single simulation output extensions.

    Returns
    -------
    :
        List of missing conditions, including random seed.
    """

    missing_conditions = []

    for condition in conditions:
        all_keys = {f"{name}_{condition['key']}_{seed:04}": seed for seed in seeds}
        manifest_keys = manifest[manifest["KEY"].isin(all_keys.keys())]

        missing_keys = set(all_keys) - set(manifest_keys["KEY"].unique())
        incomplete_keys = set(
            manifest_keys.groupby("KEY").filter(
                lambda x: len(set(extensions) - set(x["EXTENSION"])) != 0
            )["KEY"]
        )

        for key in list(missing_keys) + list(incomplete_keys):
            missing_condition = condition.copy()
            missing_condition["seed"] = all_keys[key]
            missing_conditions.append(missing_condition)

    return missing_conditions



from docker import APIClient


def terminate_docker_job(api_client: APIClient, container_id: str) -> None:
    """
    Terminate specific docker container.

    Parameters
    ----------
    api_client
        Docker API client.
    container_id
        ID of container to terminate.
    """

    api_client.stop(container=container_id, timeout=1)



from typing import Optional

from docker import DockerClient


def run_docker_command(
    client: DockerClient,
    image: str,
    command: list[str],
    volume_name: Optional[str] = None,
    environment: Optional[list] = None,
    detach: bool = False,
) -> None:
    """
    Run container from image with given command.

    Parameters
    ----------
    api_client
        Docker API client.
    image
        Docker image.
    command
        Command list passed to container.
    volume_name
        Name of the docker volume.
    environment
        List of environment variables as strings.
    detach
        True to start container and immediately return the Container object,
        False otherwise.
    """

    environment = [] if environment is None else environment
    volumes = {} if volume_name is None else {volume_name: {"bind": "/mnt", "mode": "rw"}}

    client.containers.run(
        image,
        command,
        environment=environment,
        volumes=volumes,
        auto_remove=True,
        detach=detach,
    )



from typing import Optional


def make_docker_job(name: str, image: str, environment: Optional[list[str]] = None) -> dict:
    """
    Create docker job definition.

    Environment variables are passed as strings using the following structure:

    .. code-block:: python

        [
            "envName1=envValue1",
            "envName2=envValue2",
            ...
        ]

    Parameters
    ----------
    name
        Job definition name.
    image
        Docker image.
    environment
        List of environment variables as strings.

    Returns
    -------
    :
        Job definition.
    """

    job_definition = {
        "image": image,
        "name": name,
        "volumes": ["/mnt"],
    }

    if environment is not None:
        job_definition["environment"] = environment

    return job_definition



from docker import APIClient


def remove_docker_volume(api_client: APIClient, volume_name: str) -> None:
    """
    Remove docker volume.

    Parameters
    ----------
    api_client
        Docker API client.
    volume_name
        Name of the docker volume.
    """

    api_client.remove_volume(volume_name)



from typing import Any

import boto3


def submit_fargate_task(
    name: str,
    task_definition_arn: str,
    user: str,
    cluster: str,
    security_groups: list[str],
    subnets: list[str],
    command: list[str],
    **kwargs: Any,
) -> list[str]:
    """
    Submit task to AWS Fargate.

    Parameters
    ----------
    name
        Task name.
    task_definition_arn
        Task definition ARN.
    user
        User name prefix for task name.
    cluster
        ECS cluster name.
    security_groups
        List of security groups.
    subnets
        List of subnets.
    command
        Command list passed to container.
    **kwargs
        Additional parameters for task submission. The keyword arguments are
        passed to `boto3` ECS client method `run_task`.

    Returns
    -------
    :
        Task ARN.
    """

    default_task_submission = {
        "taskDefinition": task_definition_arn,
        "capacityProviderStrategy": [
            {"capacityProvider": "FARGATE", "weight": 1},
            {"capacityProvider": "FARGATE_SPOT", "weight": 1},
        ],
        "cluster": cluster,
        "platformVersion": "LATEST",
        "count": 1,
        "networkConfiguration": {
            "awsvpcConfiguration": {
                "subnets": subnets,
                "assignPublicIp": "ENABLED",
                "securityGroups": security_groups,
            }
        },
        "overrides": {
            "containerOverrides": [
                {
                    "name": f"{user}_{name}",
                    "command": command,
                }
            ]
        },
    }

    client = boto3.client("ecs")
    task_submission = default_task_submission | kwargs
    response = client.run_task(**task_submission)

    return response["tasks"][0]["taskArn"]
